"""
Coil Calculator. Modified from NoteBook.py
Inspired by 'mini Ring Core Calculator' from  DL5SWB
"""

title = 'Coil Calculator'

# Import Pmw from this directory tree.
import sys
#sys.path[:0] = ['../../..']

import os
import math
import Tkinter
import Pmw
import CoreDB
import CoilCalcClasses

class AirCoilCalc:
    def __init__(self, parent):

	# Variables
	self.ignoreLupdate = 0
	self.ignoreNupdate = 0
	self.al = 4

	group = Pmw.Group(parent, tag_text = 'Coil')
	group.pack(fill = 'both', expand = 1)

	# N entry field
	# Entryfield for # turns
	l = Tkinter.Label(group.interior(),
		text='N:',
		anchor='e',
		width = 2
	)
	l.grid(row = 0, column = 0)
	self.turns = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 10,
		errorbackground = None,
		modifiedcommand = self.Nchanged
	)
	self.turns.grid(row=0, column=1, columnspan=2, sticky='w')

	# Entryfield for Diameter
	l = Tkinter.Label(group.interior(),
		text='d:',
		anchor='e',
		width = 2
	)
	l.grid(row = 1, column = 0)
	self.diameter = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 5,
		errorbackground = None,
		modifiedcommand = self.dchanged
	)
	self.diameter.grid(row = 1, column = 1)

	# Selection box Diameter magnitude
	mags = ('mm', 'cm')
	self.dmag = Pmw.ComboBox(group.interior(),
		labelpos = None,
		entry_width = 5,
		selectioncommand = self.dmagchanged,
		scrolledlist_items = mags
	)
	self.dmag.grid(row = 1, column = 2)
	self.dmag.selectitem(mags[0])

	# Entryfield for Length
	l = Tkinter.Label(group.interior(),
		text='l:',
		anchor='e',
		width = 2
	)
	l.grid(row = 2, column = 0)
	self.length = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 5,
		errorbackground = None,
		modifiedcommand = self.lchanged
	)
	self.length.grid(row = 2, column = 1)

	# Selection box Length magnitude
	mags = ('mm', 'cm')
	self.lenmag = Pmw.ComboBox(group.interior(),
		labelpos = None,
		entry_width = 5,
		selectioncommand = self.lmagchanged,
		scrolledlist_items = mags
	)
	self.lenmag.grid(row = 2, column = 2)
	self.lenmag.selectitem(mags[0])

	# L entry field
	# Entryfield for Inductance
	l = Tkinter.Label(group.interior(),
		text='L:',
		anchor='e',
		width = 3
	)
	l.grid(row = 0, column = 3)
	self.coilvalue = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 6,
		errorbackground = None,
		modifiedcommand = self.Lchanged
	)
	self.coilvalue.grid(row = 0, column = 4)

	# Selection box L magnitude
	mags = ('nH','uH','mH', 'H')
	self.lmag = Pmw.ComboBox(group.interior(),
		labelpos = None,
		entry_width = 4,
		selectioncommand = self.Lmagchanged,
		scrolledlist_items = mags
	)
	self.lmag.grid(row = 0, column = 5)
	self.lmag.selectitem(mags[1])

	# reactance group
	group = Pmw.Group(parent, tag_text = 'Reactance')
	group.pack(fill = 'both', expand = 1)

	# Entryfield for Frequency
	l = Tkinter.Label(group.interior(),
		text='F:',
		anchor='e',
		width = 2
	)
	l.grid(row = 0, column = 0)
	self.frequency = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 5,
		errorbackground = None,
		modifiedcommand = self.setFreq
	)
	self.frequency.grid(row=0, column=1, sticky='w')

	# Selection box frequency magnitude
	mags = ('Hz', 'KHz' , 'MHz' , 'GHz')
	self.freqmag = Pmw.ComboBox(group.interior(),
		labelpos = None,
		entry_width = 5,
		selectioncommand = self.setFreqmag,
		scrolledlist_items = mags
	)
	self.freqmag.grid(row = 0, column = 2)
	self.freqmag.selectitem(mags[2])

	# XL field
	l = Tkinter.Label(group.interior(),
		text='Xl:', anchor='e', width = 3 )
	l.grid(row = 0, column = 3)
	self.Xl = Tkinter.Label(group.interior(), width = 10, text = '')
	self.Xl.grid(row=0, column = 4, columnspan=2)

    def Nchanged(self):
	self.updateL()

    def Lchanged(self):
	self.updateN()

    def Lmagchanged(self, s):
	self.updateN()

    def dchanged(self):
	self.updateL()

    def dmagchanged(self, s):
	self.updateL()

    def lchanged(self):
	self.updateL()

    def lmagchanged(self, s):
	self.updateL()

    def setFreq(self):
	self.updateXl()

    def setFreqmag(self, s):
	self.updateXl()

    # L = N*N * d*d / ( 456*d + 1016*l)
    # L in uH, d and l in mm
    def updateL(self):
	if self.ignoreLupdate == 1:
		self.ignoreLupdate = 0
		return
	turns = self.turns.getvalue()
	if turns == '':
		self.coilvalue.setvalue('')
		return
	n = float(turns)
	diameter = self.diameter.getvalue()
	if diameter == '':
		self.coilvalue.setvalue('')
		return
	d = float(diameter)
	if self.dmag.get() == 'cm':
		d = d * 10.0
	length = self.length.getvalue()
	if length == '':
		self.coilvalue.setvalue('')
		return
	len = float(length)
	if self.lenmag.get() == 'cm':
		len = len * 10.0
	# l in uH
	l = n*n * d*d / ( 456*d + 1016*len)
	# convert to H and save
	self.l = l / 1e6
	# pretty 'print' value, L in uH
	if l > 1000000.0:
		l = l / 1000000.0
		mag = 'H'
	elif l > 1000.0:
		l = l / 1000.0
		mag = 'mH'
	elif l > 1.0:
		mag = 'uH'
	else:
		l = l * 1000.0
		mag = 'nH'
	# ignore next call to Lchanged
	self.ignoreNupdate = 1
	self.coilvalue.setvalue("%.1f" % l)
	self.lmag.selectitem(mag)
	self.updateXl()

    # N = sqrt(L * (456*d + 1016*l)) / d
    # L in uH, d and l in mm
    def updateN(self):
	if self.ignoreNupdate == 1:
		self.ignoreNupdate = 0
		return
	inductance = self.coilvalue.getvalue()
	if inductance == '':
		self.turns.setvalue('')
		return
	diameter = self.diameter.getvalue()
	if diameter == '':
		self.turns.setvalue('')
		return
	d = float(diameter)
	if self.dmag.get() == 'cm':
		d = d * 10.0
	length = self.length.getvalue()
	if length == '':
		self.turns.setvalue('')
		return
	len = float(length)
	if self.lenmag.get() == 'cm':
		len = len * 10.0
	mag = self.lmag.get()
	# convert to uH
	l = float(inductance)
	if mag == 'nH':
		l = l / 1e3
	elif mag == 'mH':
		l = l * 1e3
	elif mag == 'H':
		l = l * 1e6
	n = math.sqrt(l * (456*d + 1016*len)) / d
	# ignore next call to Nchanged
	self.ignoreLupdate = 1
	self.turns.setvalue("%.1f" % n)
	# save l in H
	self.l = l / 1e6
	self.updateXl()

    def updateXl(self):
	if self.coilvalue.get() == '':
		return
	freq = self.frequency.getvalue()
	if freq == '':
		self.Xl.configure(text='')
		return
	freqmag = self.freqmag.get()
	f = float(freq)
	if freqmag == 'KHz':
		f = f * 1e3
	elif freqmag == 'MHz':
		f = f * 1e6
	elif freqmag == 'GHz':
		f = f * 1e9
	xl = 2 * math.pi * f * self.l
	if xl > 1e9:
		xl = xl / 1e9
		u = 'Gohm'
	elif xl > 1e6:
		xl = xl / 1e6
		u = 'Mohm'
	elif xl > 1e3:
		xl = xl / 1e3
		u = 'Kohm'
	else:
		u = 'Ohm'
	self.Xl.configure(text="%.1f" % xl + u)

class NotImplemented:
    def __init__(self, parent):
	w = Tkinter.Label(parent,
		text = 'Not implemented yet',
		relief = 'sunken'
	)
	w.pack(padx = 8, pady = 8)

class CoilCalc:
    def __init__(self, parent):
	# Create and pack the NoteBook.
	notebook = Pmw.NoteBook(parent)
	notebook.pack(fill = 'both', expand = 1, padx = 10, pady = 10)

	# Add IronPowder Page
	page = notebook.add('IronPowder')
	p1 = CoilCalcClasses.IronPowderCoreCalc(page)

	# Add Ferrite page
	page = notebook.add('Ferrite')
	p2 = CoilCalcClasses.FerriteCoreCalc(page)

	# Add Unknown Core Page
	page = notebook.add('Unknown')
	group = CoilCalcClasses.UnknownCoreCalc(page)

	# Add Air Coil page
	page = notebook.add('Air Coil')
	group = AirCoilCalc(page)

	notebook.setnaturalsize()

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root)
    root.title(title)

    print "pid =" , os.getpid()

    widget = CoilCalc(root)
    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack()
    root.mainloop()
