"""
Coil Calculator Classes
"""

import os
import math
import Tkinter
import Pmw
import CoreDB

class CoreSelect:
    def __init__(self, parent, coresizes, materials):

	# Selection box for core size
	group1 = Pmw.Group(parent, tag_text = 'Core')
	group1.pack(fill = 'both')
	self.sizebox = Pmw.ComboBox(group1.interior(),
	    label_text = 'Size',
	    labelpos = 'w',
	    entry_width = 8,
	    selectioncommand = None,
	    scrolledlist_items = coresizes
	)
	self.sizebox.grid(row=0, column=0, pady=5)

	# Selection box for core material
        self.materialbox = Pmw.ComboBox(group1.interior(),
		label_text = 'Material',
		labelpos = 'w',
		entry_width = 6,
                selectioncommand = None,
		scrolledlist_items = materials
	)
	self.materialbox.grid(row = 0, column = 1)

    def setCallbacks(self,sizeCallback,materialCallback):
	self.sizebox.configure(selectioncommand = sizeCallback)
	self.materialbox.configure(selectioncommand = materialCallback)

    def setItems(self, core, material):
	self.sizebox.selectitem(core)
	self.materialbox.selectitem(material)

class CoreCoilCalc:
    def __init__(self, parent):

	# Variables
	self.ignoreLupdate = 0
	self.ignoreNupdate = 0
	self.al = 4

        group = Pmw.Group(parent, tag_text = 'Coil')
	group.pack(fill = 'both', expand = 1)

	# N entry field
	# Entryfield for # turns
	l = Tkinter.Label(group.interior(),
		text='N:',
		anchor='e',
		width = 2
        )
        l.grid(row = 0, column = 0)
	self.turns = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 10,
		errorbackground = None,
		modifiedcommand = self.Nchanged
	)
        self.turns.grid(row=0, column=1, columnspan=2, sticky='w')

	# L entry field
	# Entryfield for Inductance
	l = Tkinter.Label(group.interior(),
		text='L:',
		anchor='e',
		width = 3
        )
        l.grid(row = 0, column = 3)
	self.coilvalue = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 6,
		errorbackground = None,
		modifiedcommand = self.Lchanged
	)
	self.coilvalue.grid(row = 0, column = 4)

	# Selection box L magnitude
	mags = ('nH','uH','mH', 'H')
	self.lmag = Pmw.ComboBox(group.interior(),
                labelpos = None,
		entry_width = 4,
		selectioncommand = self.Lmagchanged,
                scrolledlist_items = mags
	)
        self.lmag.grid(row = 0, column = 5)
        self.lmag.selectitem(mags[1])

	# Entryfield for frequency
	l = Tkinter.Label(group.interior(),
		text='F:',
		anchor='e',
		width = 2
        )
        l.grid(row = 1, column = 0)
	self.frequency = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 5,
		errorbackground = None,
		modifiedcommand = self.setFreq
	)
	self.frequency.grid(row=1, column=1, sticky='w')

	# Selection box frequency magnitude
	mags = ('Hz', 'KHz' , 'MHz' , 'GHz')
	self.freqmag = Pmw.ComboBox(group.interior(),
                labelpos = None,
		entry_width = 5,
		selectioncommand = self.setFreqmag,
                scrolledlist_items = mags
	)
        self.freqmag.grid(row = 1, column = 2)
        self.freqmag.selectitem(mags[2])

	# XL field
	l = Tkinter.Label(group.interior(),
		text='Xl:', anchor='e', width = 3 )
        l.grid(row = 1, column = 3)
	self.Xl = Tkinter.Label(group.interior(), width = 10, text = '')
	self.Xl.grid(row=1, column = 4, columnspan=2)

    def Nchanged(self):
	self.updateL()

    def Lchanged(self):
	self.updateN()

    def Lmagchanged(self, s):
	self.updateN()

    def setFreq(self):
	self.updateXl()

    def setFreqmag(self, s):
	self.updateXl()

    # L = Al * n * n
    def updateL(self):
	if self.ignoreLupdate == 1:
		self.ignoreLupdate = 0
		return
	turns = self.turns.getvalue()
	if turns == '':
		self.coilvalue.setvalue('')
		return
	n = float(turns)
	# l in nH
	l = self.al * n * n
	# convert to H and save
	self.l = l / 1e9
	# pretty 'print' value
	if l > 1000000000.0:
		l = l / 1000000000.0
		mag = 'H'
	elif l > 1000000.0:
		l = l / 1000000.0
		mag = 'mH'
	elif l > 1000.0:
		l = l / 1000.0
		mag = 'uH'
	else:
		mag = 'nH'
	# ignore next call to Lchanged
	self.ignoreNupdate = 1
	self.coilvalue.setvalue("%.1f" % l)
	self.lmag.selectitem(mag)
	self.updateXl()

    # N = sqrt( L / Al )
    # Al in nH / N*N
    def updateN(self):
	if self.ignoreNupdate == 1:
		self.ignoreNupdate = 0
		return
	inductance = self.coilvalue.getvalue()
	if inductance == '':
		self.turns.setvalue('')
		return
	mag = self.lmag.get()
	# convert to nH
	l = float(inductance)
	if mag == 'uH':
		l = l * 1e3
	elif mag == 'mH':
		l = l * 1e6
	elif mag == 'H':
		l = l * 1e9
	n = math.sqrt(l / self.al)
	# ignore next call to Nchanged
	self.ignoreLupdate = 1
	self.turns.setvalue("%.1f" % n)
	# save l in H
	self.l = l / 1e9
	self.updateXl()

    def updateXl(self):
	if self.coilvalue.get() == '':
		return
	freq = self.frequency.getvalue()
	if freq == '':
		self.Xl.configure(text='')
		return
	freqmag = self.freqmag.get()
	f = float(freq)
	if freqmag == 'KHz':
		f = f * 1e3
	elif freqmag == 'MHz':
		f = f * 1e6
	elif freqmag == 'GHz':
		f = f * 1e9
	xl = 2 * math.pi * f * self.l
	if xl > 1e9:
		xl = xl / 1e9
		u = 'Gohm'
	elif xl > 1e6:
		xl = xl / 1e6
		u = 'Mohm'
	elif xl > 1e3:
		xl = xl / 1e3
		u = 'Kohm'
	else:
		u = 'Ohm'
	self.Xl.configure(text="%.1f" % xl + u)

class IronPowderCoreCalc:
    def __init__(self, parent):

	# Build selection groep
	clist = CoreDB.IronPowderSizeDB.keys()
	clist.sort(CoreDB.compare_core)
	mlist = CoreDB.IronPowderMaterialDB.keys()
	mlist.sort(CoreDB.compare_material)
	self.core = 'T50'
	self.material = '2'
	selectboxgroup = CoreSelect(parent, clist, mlist)
	selectboxgroup.setCallbacks(self.changeSize, self.changeMaterial)
	selectboxgroup.setItems(self.core, self.material)

	# Build specification section
	labelgroup = Pmw.Group(parent, tag_text = 'Specifications')
	labelgroup.pack(fill = 'both')
	labelnames = ('Colour','OD','Perm.','ID','Freq.','Hgt','Al')
	self.labels = {}
	r = 0
	c = 0
	for l in labelnames:
	    # Info name
            infoname = Tkinter.Label(labelgroup.interior(),
                text=l + ':', anchor='e', width = 8 )
            infoname.grid(row=r, column=c)
	    # Info fields
            info = Tkinter.Label(labelgroup.interior(),
		text = '', width = 12, anchor='e', )
            info.grid(row=r, column=c+1)
	    # key = labelname, value = (labelname, infofield, row, column)
	    self.labels[l] = (l, info, r, c+1)
	    c += 2
	    if c > 3:
		r += 1
		c = 0
	# Error field
	self.errortext = Tkinter.Label(labelgroup.interior(),
		    text = '', width = 20, anchor='w', )
	self.errortext.grid(row=r, column=c, columnspan=2)

	# Calculate Coil group
	self.coilgroup = CoreCoilCalc(parent)

	# Fill core specifications
	self.update()

    def update(self):
	self.labels['Colour'][1].configure(
		text = CoreDB.IronPowderMaterialDB[self.material][3])
	self.labels['Perm.'][1].configure(
		text = CoreDB.IronPowderMaterialDB[self.material][0])
	self.labels['Freq.'][1].configure(
		text = '%d .. %d MHz' % (
			CoreDB.IronPowderMaterialDB[self.material][1],
			CoreDB.IronPowderMaterialDB[self.material][2])
		)
	self.labels['OD'][1].configure(
		text = '%.3f in' % CoreDB.IronPowderSizeDB[self.core][0])
	self.labels['ID'][1].configure(
		text = '%.3f in' % CoreDB.IronPowderSizeDB[self.core][1])
	self.labels['Hgt'][1].configure(
		text = '%.3f in' % CoreDB.IronPowderSizeDB[self.core][2])
	try:
	    self.al = CoreDB.IronPowderAlDB[self.core + '-' + self.material]
	    self.labels['Al'][1].configure(
		text = str(self.al) + ' nH', foreground='black')
	    self.errortext.configure(text = '')
	except:
	    self.al = 0
	    self.labels['Al'][1].configure(
		text = str(self.al), foreground='black')
	    self.errortext.configure(
		text = 'Core does not exist', foreground= 'red')

	# Coilgroup needs Al
	self.coilgroup.al = self.al
	self.coilgroup.updateL()

    def changeSize(self, s):
	self.core = s
	self.update()

    def changeMaterial(self, s):
	self.material = s
	self.update()

class FerriteCoreCalc:
    def __init__(self, parent):

	# Build selection groep
	clist = CoreDB.FerriteSizeDB.keys()
	clist.sort(CoreDB.compare_core)
	mlist = CoreDB.FerriteMaterialDB.keys()
	mlist.sort(CoreDB.compare_material)
	self.core = 'FT50'
	self.material = '43'
	selectboxgroup = CoreSelect(parent, clist, mlist)
	selectboxgroup.setCallbacks(self.changeSize, self.changeMaterial)
	selectboxgroup.setItems(self.core, self.material)

	# Build specification section
	labelgroup = Pmw.Group(parent, tag_text = 'Specifications')
	labelgroup.pack(fill = 'both')
	# Label name , columns
	labelnames = (	('Frequency Ranges',2),	('OD',1),
			('Resonant',1),		('ID',1),
			('Wide Band',1),	('Hgt',1),
			('Choke',1),		('Al',1),
			('Perm.',1),
		)
	self.labels = {}
	r = 0
	c = 0
	for l in labelnames:
	    if l[1] == 1:
	        # Info name
                infoname = Tkinter.Label(labelgroup.interior(),
		    text=l[0] + ':', anchor='e', width = 9 )
                infoname.grid(row=r, column=c, columnspan=l[1])
		# Info field
		info = Tkinter.Label(labelgroup.interior(),
		    text = '', width = 12, anchor='e', )
		info.grid(row=r, column=c+1)
	        # key = labelname, value = (labelname, infofield, row, column)
	        self.labels[l[0]] = (l[0], info, r, c+1)
	    else:
	        # Info name
                infoname = Tkinter.Label(labelgroup.interior(),
		    text=l[0], anchor='e', width = 20 )
                infoname.grid(row=r, column=c, columnspan=l[1])
	    c += 2
	    if c > 3:
		r += 1
		c = 0

	# Error field
	self.errortext = Tkinter.Label(labelgroup.interior(),
		    text = '', width = 20, anchor='w', )
	self.errortext.grid(row=r, column=c, columnspan=2)

	# Calculate Coil group
	self.coilgroup = CoreCoilCalc(parent)

	# Fill core specifications
	self.update()

    def update(self):
	self.labels['Perm.'][1].configure(
		text = CoreDB.FerriteMaterialDB[self.material][0])
	self.labels['Resonant'][1].configure(
		text = '%d .. %d MHz' % (
			CoreDB.FerriteMaterialDB[self.material][1],
			CoreDB.FerriteMaterialDB[self.material][2])
		)
	self.labels['Wide Band'][1].configure(
		text = '%d..%d MHz' % (
			CoreDB.FerriteMaterialDB[self.material][3],
			CoreDB.FerriteMaterialDB[self.material][4])
		)
	if CoreDB.FerriteMaterialDB[self.material][6] > 0:
	    self.labels['Choke'][1].configure(
		text = '%d .. %d MHz' % (
			CoreDB.FerriteMaterialDB[self.material][5],
			CoreDB.FerriteMaterialDB[self.material][6])
		)
	else:
	    self.labels['Choke'][1].configure(
		text = ' > %d MHz' % (
			CoreDB.FerriteMaterialDB[self.material][5])
		)
	self.labels['OD'][1].configure(
		text = '%.3f in' % CoreDB.FerriteSizeDB[self.core][0])
	self.labels['ID'][1].configure(
		text = '%.3f in' % CoreDB.FerriteSizeDB[self.core][1])
	self.labels['Hgt'][1].configure(
		text = '%.3f in' % CoreDB.FerriteSizeDB[self.core][2])
	try:
	    self.al = CoreDB.FerriteAlDB[self.core + '-' + self.material]
	    self.labels['Al'][1].configure(
		text = str(self.al) + ' nH', foreground='black')
	    self.errortext.configure(text = '')
	except:
	    self.al = 0
	    self.labels['Al'][1].configure(
		text = str(self.al), foreground='black')
	    self.errortext.configure(
		text = 'Core does not exist', foreground= 'red')

	# Coilgroup needs Al
	self.coilgroup.al = self.al
	self.coilgroup.updateL()

    def changeSize(self, s):
	self.core = s
	self.update()

    def changeMaterial(self, s):
	self.material = s
	self.update()

# TODO: implement tick box Al from test coil / Al from input
# If from test coiul Al should be label, else Al from  Pmw.EntryField
class UnknownCoreCalc:
    def __init__(self, parent):

	# Input area for test coil data
        group = Pmw.Group(parent, tag_text = 'Test Coil data')
	group.pack(fill = 'both', expand = 1)

	# N entry field
	# Entryfield for # turns
	l = Tkinter.Label(group.interior(),
		text='N:',
		anchor='e',
		width = 2
        )
        l.grid(row = 0, column = 0)
	self.turns = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 10,
		errorbackground = None,
		modifiedcommand = self.Nchanged
	)
        self.turns.grid(row=0, column=1, columnspan=2, sticky='w')

	# L entry field
	# Entryfield for Inductance
	l = Tkinter.Label(group.interior(),
		text='L:',
		anchor='e',
		width = 3
        )
        l.grid(row = 0, column = 3)
	self.coilvalue = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 5,
		errorbackground = None,
		modifiedcommand = self.Lchanged
	)
	self.coilvalue.grid(row = 0, column = 4)

	# Selection box L magnitude
	mags = ('nH','uH','mH', 'H')
	self.lmag = Pmw.ComboBox(group.interior(),
                labelpos = None,
		entry_width = 4,
		selectioncommand = self.LmagChanged,
                scrolledlist_items = mags
	)
        self.lmag.grid(row = 0, column = 5)
        self.lmag.selectitem(mags[1])

	# Output area for parameters
        group = Pmw.Group(parent, tag_text = 'Specifications')
	group.pack(fill = 'both', expand = 1)

#	# Choose between test coil data or direct input
#	self.use_test_coil = 0
#	cb = Tkinter.Checkbutton(group.interior(),
#		text = 'Use test coil data',
#		wraplength=100,
#		variable=self.use_test_coil,
#	)
#	cb.grid(row=0, column=0)

	# Al
	l = Tkinter.Label(group.interior(),
                text='Al:', anchor='e', width = 3 )
        l.grid(row=0, column=1)
        self.Al = Tkinter.Label(group.interior(), width = 10, text = '')
        self.Al.grid(row=0, column=2)

	# Calculate Coil group
	self.coilgroup = CoreCoilCalc(parent)

    def Nchanged(self):
	self.update()

    def Lchanged(self):
	self.update()

    def LmagChanged(self,s):
	self.update()

    def update(self):
	turns = self.turns.get()
	if turns == '':
		return
	n = float(turns)
	inductance = self.coilvalue.get()
	if inductance == '':
		return
	l = float(inductance)
	mag = self.lmag.get()
	# convert to nH
	if mag == 'uH':
		l = l * 1e3
	elif mag == 'mH':
		l = l * 1e6
	elif mag == 'H':
		l = l * 1e9

	# Al = L / (n * n)
	try:
		self.al = l / (n * n)
		self.Al.configure(text=("%.1f" % self.al) + ' nH')
	except:
		self.Al.configure(text='')

	# Propagate to CoilCalc window
	self.coilgroup.al = self.al
	self.coilgroup.updateL()

